// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:supabase_flutter/supabase_flutter.dart';
import 'package:wyatt_authentication/wyatt_authentication.dart';

class AuthenticationSupabasePlatform<A>
    extends AuthenticationPlatform<AuthResponse, A> {
  AuthenticationSupabasePlatform({
    required this.accountMapper,
    SupabaseClient? supabase,
  }) : _supabase = supabase ?? Supabase.instance.client;

  final SupabaseClient _supabase;
  final A? Function(User? user) accountMapper;

  @override
  String get name => 'Supabase';

  @override
  Future<AuthResponse> signIn<T extends SignInProvider>(
    T provider,
  ) async {
    if (provider is EmailPasswordAuthProvider) {
      return _supabase.auth.signInWithPassword(
        email: provider.email,
        password: provider.password,
      );
    }
    if (provider is GoogleSignInAuthProvider) {
      return _supabase.auth.signInWithIdToken(
        provider: OAuthProvider.google,
        idToken: provider.idToken,
        accessToken: provider.accessToken,
      );
    }
    throw UnsupportedError(
      'The provider $provider is not supported by $name',
    );
  }

  @override
  Future<void> signOut() async {
    await _supabase.auth.signOut();
  }

  @override
  Future<AuthResponse> signUp<T extends SignUpProvider>(T provider) async {
    if (provider is EmailPasswordAuthProvider) {
      return _supabase.auth.signUp(
        email: provider.email,
        password: provider.password,
      );
    }
    throw UnsupportedError(
      'The provider $provider is not supported by $name',
    );
  }

  @override
  Future<A?> get currentAccount async =>
      accountMapper(_supabase.auth.currentUser);

  @override
  Stream<A?> get accountChanges => _supabase.auth.onAuthStateChange.map(
        (authState) => accountMapper(authState.session?.user),
      );
}
